==> japhar-0.04.tar.gz: japhar-0.04/lib/libnative/java.lang/runtime.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   runtime.c -- native methods for java/lang/Runtime.
   Created: Chris Toshok <toshok@hungry.com>, 24-Oct-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "jniint.h"
#include "log.h"
#include "dynamic_loading.h"
#include "interp.h"
#include <assert.h>
#include <sys/time.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define MYLOG "Native"

JNIEXPORT void JNICALL
Java_java_lang_Runtime_exitInternal(JNIEnv *env,
				    jobject obj,
				    jint exit_status)
{
  HungryJNIEnv *henv = (HungryJNIEnv*)env;

  if (henv->_vm->_args.exit)
    (*henv->_vm->_args.exit)(exit_status);
  else
    exit(exit_status);
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_runFinalizersOnExit0(JNIEnv *env,
					    jclass cls,
					    jboolean flag)
{
  HungryJNIEnv *henv = (HungryJNIEnv*)env;

  henv->_vm->_finalizers_on_exit = flag;
}

JNIEXPORT jobject JNICALL
Java_java_lang_Runtime_execInternal(JNIEnv *env,
				    jobject obj,
				    jobjectArray cmdarray,
				    jobjectArray envp)
{
  jclass unixproc = (*env)->FindClass(env, "java/lang/UNIXProcess");
  jmethodID ctor = (*env)->GetMethodID(env, unixproc, "<init>", "([Ljava/lang/String,[Ljava/lang/String)V");
  jobject new_proc;

  new_proc = (*env)->NewObject(env, unixproc, ctor, cmdarray, envp);

  return new_proc;
}

JNIEXPORT jlong JNICALL
Java_java_lang_Runtime_freeMemory(JNIEnv *env,
				  jobject obj)
{
  (*env)->FatalError(env, "Java_java_lang_Runtime_freeMemory not implemented");
  return 0;
}

JNIEXPORT jlong JNICALL
Java_java_lang_Runtime_totalMemory(JNIEnv *env,
				   jobject obj)
{
  (*env)->FatalError(env, "Java_java_lang_Runtime_totalMemory not implemented");
  return 0;
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_gc(JNIEnv *env,
			  jobject obj)
{
  HungryJNIEnv *henv = (HungryJNIEnv*)env;

  GC_runCollector((JavaVM*)henv->_vm);
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_runFinalization(JNIEnv *env,
				       jobject obj)
{
  HungryJNIEnv *henv = (HungryJNIEnv*)env;

  GC_runFinalizers((JavaVM*)henv->_vm);
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_traceInstructions(JNIEnv *env,
					 jobject obj,
					 jboolean on)
{
  (*env)->FatalError(env, "Java_java_lang_Runtime_traceInstructions not implemented");
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_traceMethodCalls(JNIEnv *env,
					jobject obj,
					jboolean on)
{
  (*env)->FatalError(env, "Java_java_lang_Runtime_traceMethodCalls not implemented");
}

JNIEXPORT jstring JNICALL
Java_java_lang_Runtime_initializeLinkerInternal(JNIEnv *env,
						jobject obj)
{
  jstring ld_library_path;
  char *ld_env = getenv("LD_LIBRARY_PATH");
  char *ld_env_with_internal = NULL;

  /* LIBDIR is always in DLL path */
  if (ld_env == NULL)
    ld_env = LIBDIR;
  else
    {
      ld_env_with_internal = malloc(strlen(ld_env) + strlen(":" LIBDIR) + 1);
      strcpy(ld_env_with_internal, ld_env);
      strcat(ld_env_with_internal, ":" LIBDIR);
      ld_env = ld_env_with_internal;
    }

  ld_library_path = (*env)->NewStringUTF(env, ld_env);

  ld_library_path = (*env)->NewGlobalRef(env, ld_library_path);

  if (NULL != ld_env_with_internal)
    free(ld_env_with_internal);

  return ld_library_path;
}

JNIEXPORT jstring JNICALL
Java_java_lang_Runtime_buildLibName(JNIEnv *env,
				    jobject obj,
				    jobject pathname,
				    jobject filename)
{
  jstring lib_name;
  const jbyte* filename_bytes;
  jint filename_length;
  const jbyte* pathname_bytes;
  jint pathname_length;
  char *filename_str;
  char *pathname_str;
  char *libname_str;

  filename_bytes = (*env)->GetStringUTFChars(env, filename, NULL);
  filename_length = (*env)->GetStringUTFLength(env, filename) + 1;
  filename_str = (char*)malloc(filename_length * sizeof(char));

  strncpy(filename_str, (char*)filename_bytes, filename_length);

  (*env)->ReleaseStringUTFChars(env, filename, filename_bytes);

  pathname_bytes = (*env)->GetStringUTFChars(env, pathname, NULL);
  pathname_length = (*env)->GetStringUTFLength(env, pathname) + 1;
  pathname_str = (char*)malloc(pathname_length * sizeof(char));

  strncpy(pathname_str, (char*)pathname_bytes, pathname_length);

  (*env)->ReleaseStringUTFChars(env, pathname, pathname_bytes);

  libname_str = (char*)malloc(strlen(pathname_str) + strlen(filename_str) + strlen("libjaphar_.so") + 1);

  sprintf(libname_str, "%s/libjaphar_%s.so", pathname_str, filename_str);

  free(pathname_str);
  free(filename_str);

  lib_name = (*env)->NewStringUTF(env, libname_str);
  lib_name = (*env)->NewGlobalRef(env, lib_name);

  return lib_name;
}

JNIEXPORT jint JNICALL
Java_java_lang_Runtime_loadFileInternal(JNIEnv *env,
					jobject obj,
					jobject filename)
{
  DLL_LibHandle handle;
  int filename_length;
  const jbyte *filename_bytes;
  char *filename_str;

  filename_bytes = (*env)->GetStringUTFChars(env, filename, NULL);

  filename_length = (*env)->GetStringUTFLength(env, filename) + 1;
  filename_str = (char*)malloc(filename_length * sizeof(char));

  strncpy(filename_str, (char*)filename_bytes, filename_length);

  (*env)->ReleaseStringUTFChars(env, filename, filename_bytes);
  
  handle = DLL_load(filename_str);

  free(filename_str);

  if (handle)
    return 1;
  else
    return 0;
}

==> japhar-0.05.tar.gz: japhar-0.05/lib/libnative/java.lang/runtime.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   runtime.c -- native methods for java/lang/Runtime.
   Created: Chris Toshok <toshok@hungry.com>, 24-Oct-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "jniint.h"
#include "log.h"
#include "dynamic_loading.h"
#include "interp.h"
#include "compat.h"
#include <assert.h>
#include <sys/time.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define MYLOG "Native"

JNIEXPORT void JNICALL
Java_java_lang_Runtime_exitInternal(JNIEnv *env,
				    jobject obj,
				    jint exit_status)
{
  HungryJNIEnv *henv = (HungryJNIEnv*)env;

  if (henv->_vm->_args.exit)
    (*henv->_vm->_args.exit)(exit_status);
  else
    exit(exit_status);
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_runFinalizersOnExit0(JNIEnv *env,
					    jclass cls,
					    jboolean flag)
{
  HungryJNIEnv *henv = (HungryJNIEnv*)env;

  henv->_vm->_finalizers_on_exit = flag;
}

JNIEXPORT jobject JNICALL
Java_java_lang_Runtime_execInternal(JNIEnv *env,
				    jobject obj,
				    jobjectArray cmdarray,
				    jobjectArray envp)
{
  jclass unixproc = (*env)->FindClass(env, "java/lang/UNIXProcess");
  jmethodID ctor = (*env)->GetMethodID(env, unixproc, "<init>", "([Ljava/lang/String,[Ljava/lang/String)V");
  jobject new_proc;

  new_proc = (*env)->NewObject(env, unixproc, ctor, cmdarray, envp);

  return new_proc;
}

JNIEXPORT jlong JNICALL
Java_java_lang_Runtime_freeMemory(JNIEnv *env,
				  jobject obj)
{
  (*env)->FatalError(env, "Java_java_lang_Runtime_freeMemory not implemented");
  return 0;
}

JNIEXPORT jlong JNICALL
Java_java_lang_Runtime_totalMemory(JNIEnv *env,
				   jobject obj)
{
  (*env)->FatalError(env, "Java_java_lang_Runtime_totalMemory not implemented");
  return 0;
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_gc(JNIEnv *env,
			  jobject obj)
{
  HungryJNIEnv *henv = (HungryJNIEnv*)env;

  GC_runCollector((JavaVM*)henv->_vm);
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_runFinalization(JNIEnv *env,
				       jobject obj)
{
  HungryJNIEnv *henv = (HungryJNIEnv*)env;

  GC_runFinalizers((JavaVM*)henv->_vm);
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_traceInstructions(JNIEnv *env,
					 jobject obj,
					 jboolean on)
{
  (*env)->FatalError(env, "Java_java_lang_Runtime_traceInstructions not implemented");
}

JNIEXPORT void JNICALL
Java_java_lang_Runtime_traceMethodCalls(JNIEnv *env,
					jobject obj,
					jboolean on)
{
  (*env)->FatalError(env, "Java_java_lang_Runtime_traceMethodCalls not implemented");
}

JNIEXPORT jstring JNICALL
Java_java_lang_Runtime_initializeLinkerInternal(JNIEnv *env,
						jobject obj)
{
  jstring ld_library_path;
  char *ld_env = getenv("LD_LIBRARY_PATH");
  char *ld_env_with_internal = NULL;

  /* LIBDIR is always in DLL path */
  if (ld_env == NULL)
    ld_env = LIBDIR;
  else
    {
      ld_env_with_internal = malloc(strlen(ld_env) + strlen(":" LIBDIR) + 1);
      strcpy(ld_env_with_internal, ld_env);
      strcat(ld_env_with_internal, ":" LIBDIR);
      ld_env = ld_env_with_internal;
    }

  ld_library_path = (*env)->NewStringUTF(env, ld_env);

  ld_library_path = (*env)->NewGlobalRef(env, ld_library_path);

  if (NULL != ld_env_with_internal)
    free(ld_env_with_internal);

  return ld_library_path;
}

JNIEXPORT jstring JNICALL
Java_java_lang_Runtime_buildLibName(JNIEnv *env,
				    jobject obj,
				    jobject pathname,
				    jobject filename)
{
  jstring lib_name;
  const jbyte* filename_bytes;
  jint filename_length;
  int libname_len;
  const jbyte* pathname_bytes;
  jint pathname_length;
  char *filename_str;
  char *pathname_str;
  char *libname_str;

  filename_bytes = (*env)->GetStringUTFChars(env, filename, NULL);
  filename_length = (*env)->GetStringUTFLength(env, filename) + 1;
  filename_str = (char*)malloc(filename_length * sizeof(char));

  strncpy(filename_str, (char*)filename_bytes, filename_length);

  (*env)->ReleaseStringUTFChars(env, filename, filename_bytes);

  pathname_bytes = (*env)->GetStringUTFChars(env, pathname, NULL);
  pathname_length = (*env)->GetStringUTFLength(env, pathname) + 1;
  pathname_str = (char*)malloc(pathname_length * sizeof(char));

  strncpy(pathname_str, (char*)pathname_bytes, pathname_length);

  (*env)->ReleaseStringUTFChars(env, pathname, pathname_bytes);
  libname_len = strlen(pathname_str) + strlen(filename_str) + strlen("/libjaphar_.so") + 1;
  libname_str = (char*)malloc(libname_len);

  snprintf(libname_str, libname_len, "%s/libjaphar_%s.so",
	   pathname_str, filename_str);

  free(pathname_str);
  free(filename_str);

  lib_name = (*env)->NewStringUTF(env, libname_str);
  lib_name = (*env)->NewGlobalRef(env, lib_name);

  free(libname_str);

  return lib_name;
}

JNIEXPORT jint JNICALL
Java_java_lang_Runtime_loadFileInternal(JNIEnv *env,
					jobject obj,
					jobject filename)
{
  DLL_LibHandle handle;
  int filename_length;
  const jbyte *filename_bytes;
  char *filename_str;

  filename_bytes = (*env)->GetStringUTFChars(env, filename, NULL);

  filename_length = (*env)->GetStringUTFLength(env, filename) + 1;
  filename_str = (char*)malloc(filename_length * sizeof(char));

  strncpy(filename_str, (char*)filename_bytes, filename_length);

  (*env)->ReleaseStringUTFChars(env, filename, filename_bytes);
  
  handle = DLL_load(filename_str);

  free(filename_str);

  if (handle)
    return 1;
  else
    return 0;
}
